@@ -1,7 +1,7 @@
 /*
  * libev poll fd activity backend
  *
- * Copyright (c) 2007 Marc Alexander Lehmann <libev@schmorp.de>
+ * Copyright (c) 2007,2008 Marc Alexander Lehmann <libev@schmorp.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modifica-
@@ -42,6 +42,8 @@
 void inline_size
 pollidx_init (int *base, int count)
 {
+  /* consider using memset (.., -1, ...), which is pratically guarenteed
+   * to work on all systems implementing poll */
   while (count--)
     *base++ = -1;
 }
@@ -87,7 +89,11 @@ static void
 poll_poll (EV_P_ ev_tstamp timeout)
 {
   struct pollfd *p;
-  int res = poll (polls, pollcnt, (int)ceil (timeout * 1000.));
+  int res;
+  
+  EV_RELEASE_CB;
+  res = poll (polls, pollcnt, (int)ceil (timeout * 1000.));
+  EV_ACQUIRE_CB;
 
   if (expect_false (res < 0))
     {
@@ -96,24 +102,24 @@ poll_poll (EV_P_ ev_tstamp timeout)
       else if (errno == ENOMEM && !syserr_cb)
         fd_enomem (EV_A);
       else if (errno != EINTR)
-        syserr ("(libev) poll");
+        ev_syserr ("(libev) poll");
     }
+  else
+    for (p = polls; res; ++p)
+      if (expect_false (p->revents)) /* this expect is debatable */
+        {
+          --res;
 
-  for (p = polls; res; ++p)
-    if (expect_false (p->revents)) /* this expect is debatable */
-      {
-        --res;
-
-        if (expect_false (p->revents & POLLNVAL))
-          fd_kill (EV_A_ p->fd);
-        else
-          fd_event (
-            EV_A_
-            p->fd,
-            (p->revents & (POLLOUT | POLLERR | POLLHUP) ? EV_WRITE : 0)
-            | (p->revents & (POLLIN | POLLERR | POLLHUP) ? EV_READ : 0)
-          );
-      }
+          if (expect_false (p->revents & POLLNVAL))
+            fd_kill (EV_A_ p->fd);
+          else
+            fd_event (
+              EV_A_
+              p->fd,
+              (p->revents & (POLLOUT | POLLERR | POLLHUP) ? EV_WRITE : 0)
+              | (p->revents & (POLLIN | POLLERR | POLLHUP) ? EV_READ : 0)
+            );
+        }
 }
 
 int inline_size
